/*  Wiring:
  A4 - SDA
  A5 - SCL
  10 - CS
  11 - MOSI
  12 - MISO
  13 - SCK
*/
#include <Wire.h>
#include "Adafruit_TCS34725.h"  //Library for th RGB Sensor

#include <SPI.h>
#include <SD.h>                 //Library for the SD Card Module
//PIN ASSIGNMENT
const int chipSelect = 10;       //Control Pin for the SD Card Module
int trigger = 2;

Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_50MS, TCS34725_GAIN_4X);
//VARIABLES DEFINITION
String fileName = "perceptron";
String extension = ".csv";
String file = fileName + extension;
int samples = 0;
int resultado = 0;
//PERCEPTRON PARAMETERS
float w[4] = { 103469, -1136.342623965437, 1770.317859324734, 274.6130405099033};
// these were calculated with Matlab and copied into a vector
void setup() {
  Serial.begin(9600); //inicia la comunicación Serial
  pinMode(trigger, INPUT_PULLUP);
  Serial.println("Perceptron Test");
  loadRGB();  //RGB Sensor Start Sequence
  loadSD(); //SD Card module start sequence
 
}

void loop() {
  String dataString = ""; //Variable for writing data into the file
  float red, green, blue; //One variable for each colour
  if (!digitalRead(trigger)) {
    resultado=0;
    samples += 1;
    //We use the RGB Sensor
    tcs.setInterrupt(false);    //Turn the LED on
    delay(60);                  //It takes 50ms to read
    tcs.setInterrupt(true);    //Turn the LED off
    tcs.getRGB(&red, &green, &blue);
    Serial.print("S:\t");      Serial.print(samples);
    Serial.print("\tR:\t");    Serial.print(red);
    Serial.print("\tG:\t");    Serial.print(green);
    Serial.print("\tB:\t");    Serial.println(blue);
    //Once the values are obtained, we use our trained perceptron
    Serial.print("Result: ");
    resultado = perceptron(red, green, blue);
    if (resultado == -1) {
      Serial.println("Potato");
    }
    else if (resultado == 1) {
      Serial.println("Lemon");
    }
  }
}
void loadRGB() {
  Serial.println("Buscando sensor");
  delay(100);
  Serial.println("Buscando sensor.");
  delay(100);
  Serial.println("Buscando sensor..");
  delay(100);
  Serial.println("Buscando sensor...");
  delay(100);
  if (tcs.begin()) {
    Serial.println("Sensor Detectado");
  } else {
    Serial.println("No se encontró el sensor :(");
    while (1);
  }
}
void loadSD() {
  Serial.println("Iniciando el módulo SD");
  delay(100);
  Serial.println("Iniciando el módulo SD.");
  delay(100);
  Serial.println("Iniciando el módulo SD..");
  delay(100);
  Serial.println("Iniciando el módulo SD...");
  delay(100);
   // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) {
    Serial.println("SD Card failed, or not present");
    // don't do anything more:
    while (1);
  }
  Serial.println("SD Card initialized");
}
int perceptron(float r, float g, float b) {
  float u = 0;
  int y = 0;
  float inputs[3] = {r, g, b};
  for (int i = 0; i < 4; i++) {
    for (int j = 0; i < 3; i++) {
      u = w[i] * inputs[j];
      y = biStepSignal(u);
      return y;
    }
  }
}
int biStepSignal( float u) {
  if (u < 0) {
    return -1;
  }
  else if(u>=0){
    return 1;
  }
}
